﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WSCF.RevCtrl.Test.IService;

namespace WSCF.RevCtrl.Test.Client
{
    public class Customer : ICustomerService
    {
        const int MsgsCap = 100;

        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// 用户名称
        /// </summary>
        public string Name { get; set; } = "NO Name";


        /// <summary>
        /// 不接收群聊信息
        /// </summary>
        public bool NotReceiveGroupChat { get; set; }


        /// <summary>
        /// 全部聊天记录
        /// </summary>
        public ObservableCollection<string> Msgs { get; } = new ObservableCollection<string>();

        IChatService chatService;
        string filePath = "customer.json";
        public void Init(IChatService chatService) 
        {
            this.chatService = chatService;

            //加载 用户名称
            CustomerJsonDb.Load(this, filePath);

            //用户名称 被改变，立刻保存
            this.PropertyChanged += (s, e) =>
            {
                if (e.PropertyName == nameof(Name))
                {
                    CustomerJsonDb.Save(this, filePath);
                }
            };

            this.chatService.HavingNewMsgInGroupChat += ChatService_HavingNewMsgInGroupChat;

            // 当 chatService 是远程服务的，检测服务是否已经连接好，properties 都已经同步完成？
            GetRecentMsgs();

            //当再次断开后，每次连接上都应该重新获取记录
            this.chatService.PropertyChanged += (s, e) =>
            {
                if (e.PropertyName == nameof(Reflect_SeviceClient.IsSynced))
                {
                    GetRecentMsgs();
                }
            };
        }
        void GetRecentMsgs() 
        {
            var client = chatService as Reflect_SeviceClient;
            if (client.IsSynced)
            {
                //获取全部群聊信息
                this.chatService.GetRecentMsgs((asyncContext, retData) =>
                {
                    var reponse = retData as GetRecentMsgsReponse;
                    Msgs.Clear();
                    AddMsgs(reponse.Msgs);
                }, null);
            }
        }
        private void ChatService_HavingNewMsgInGroupChat(object sender, EventArgs e)
        {
            if (NotReceiveGroupChat)
                return;

            var reponse = e as HavingNewMsgInGroupChatEventArgs;

            AddMsg(reponse.msg);

        }

        /// 服务器把某人的私聊发给用户
        /// </summary>
        /// <param name="srcGuid">某人的全局唯一ID</param>
        /// <param name="srcName">某人名字</param>
        /// <param name="time">消息的时间</param>
        /// <param name="msg">消息</param>
        public void SendMsgToPrivateChat(string srcGuid, string srcName, DateTime time, string msg) 
        {
            string text = $"{time} [{srcName}] to ** me **: {msg}";
            AddMsg(text);
        }

        void AddMsg(string msg) 
        {

            Msgs.Add(msg);
            //最多显示10条记录
            while (Msgs.Count > MsgsCap)
                Msgs.RemoveAt(0);
        }
        
        void AddMsgs(IEnumerable<string> msgs)
        {
            foreach(var msg in msgs)
                Msgs.Add(msg);
            //最多显示10条记录
            while (Msgs.Count > MsgsCap)
                Msgs.RemoveAt(0);
        }

        public void ClearMsgs() 
        {
            Msgs.Clear();
        }
        
        protected void NotifyPropertyChanged(string propertyName) {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    
        
    }

    /// <summary>
    /// customer 的数据保存
    /// </summary>
    public class CustomerJsonDb 
    {
        public string Name;
        public static bool Save(Customer customer, string filePath) 
        {
            try
            {
                var jsonDb = new CustomerJsonDb();
                jsonDb.Name = customer.Name;

                string json = Newtonsoft.Json.JsonConvert.SerializeObject(jsonDb, Newtonsoft.Json.Formatting.Indented);

                File.WriteAllText(filePath,json);
                return true;
            }
            catch
            {

                return false;
            }
        }
        public static bool Load(Customer customer, string filePath) 
        {
            if (!File.Exists(filePath))
                return false;
            try
            {
                string json = File.ReadAllText(filePath);
                var jsonDb = Newtonsoft.Json.JsonConvert.DeserializeObject<CustomerJsonDb>(json);
                customer.Name = jsonDb.Name;
                return true;
            }
            catch {

                return false;
            }
        }
    }
}
