﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WSCF.RevCtrl.Test.IService
{
    /// <summary>
    /// 聊天室服务
    /// </summary>
    public interface IChatService:INotifyPropertyChanged
    {
        //当前全部用户数量
        int CustomerCount { get; }

        /// <summary>
        /// 全部用户列表
        /// </summary>
        [PropertyPush]
        CustomerInfo[] CustomerInfos { get; }

        /// <summary>
        /// 在群聊室，有新的消息
        /// </summary>
        [Push(typeof(HavingNewMsgInGroupChatEventArgs))]
        event EventHandler HavingNewMsgInGroupChat;

        /// <summary>
        /// 获取近期群聊信息(共10条)
        /// </summary>
        /// <param name="asyncDelegate"></param>
        /// <param name="asyncContext"></param>
        [Call(typeof(GetRecentMsgsReponse))]
        void GetRecentMsgs(AsyncCBHandler asyncDelegate, object asyncContext);

        /// <summary>
        /// 发送消息到群聊
        /// </summary>
        /// <param name="msg"></param>
        [Call(typeof(SendMsgToGroupChatReponse))] 
        void SendMsgToGroupChat(string msg, AsyncCBHandler asyncDelegate, object asyncContext);

        /// <summary>
        /// 私聊某人
        /// </summary>
        /// <param name="targetGuid"></param>
        /// <param name="msg"></param>
        [Call(typeof(SendMsgToPrivateChatReponse))]
        void SendMsgToPrivateChat(string targetGuid, string msg, AsyncCBHandler asyncDelegate, object asyncContext);


    }
    /// <summary>
    /// 什么内容都没有，就是服务收到后信息后，返回一下而已
    /// </summary>
    public class SendMsgToGroupChatReponse
    {

    }


    public class SendMsgToPrivateChatReponse
    {
        /// <summary>
        /// 发送成功
        /// </summary>
        public bool isSuccess;
    }
    public class HavingNewMsgInGroupChatEventArgs:EventArgs
    {
        public string msg;
    }
    public class GetRecentMsgsReponse
    {
        public string[] Msgs;
    }

    public class CustomerInfo : INotifyPropertyChanged
    {
        /// <summary>
        /// 全局唯一码，用于识别用户
        /// </summary>
        public string Guid { get; set; }


        /// <summary>
        /// 用户名称
        /// </summary>
        public string Name { get; set; }

       

        public event PropertyChangedEventHandler PropertyChanged;
    }
}
