﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WebSocketSharp;
using WebSocketSharp.Server;
using static WSCF.Reflect_OBJ_INTERFACE;

namespace WSCF.RevCtrl
{
    public class RevCtrl_Proxy : Reflect_Proxy
    {
        public bool IsConnected { get; private set; }


        IRevCtrlAdmin admin;
        /// <summary>
        /// 服务器中，反向控制客户端，对于客户端每个服务的客户端
        /// </summary>
        Dictionary<string, WsServiceClient> Clients = new Dictionary<string, WsServiceClient>();


        public RevCtrl_Proxy()
        {

        }
        public void Init(Type interfaceType, IRevCtrlAdmin admin) 
        {
            this.admin = admin;
            base.Init(interfaceType, admin);
        }

        protected override void OnOpen()
        {
            base.OnOpen();
            IsConnected = true;
            admin.OnOpen(this);

            foreach (var kv in Clients)
            {
                var client = kv.Value;
                client.OnOpen();
            }
        }

        protected override void OnClose(CloseEventArgs e)
        {
            base.OnClose(e);
            IsConnected = false;

            admin.OnClose(this);

            foreach (var kv in Clients)
            {
                var client = kv.Value;
                client.OnClose();
            }
        }

        protected override void OnMessage(MessageEventArgs e)
        {
            if (!e.IsText)
                return;
            string json = e.Data;

            var pkgData = Newtonsoft.Json.JsonConvert.DeserializeObject<PkgData>(json);


            if (string.IsNullOrEmpty(pkgData.rcsn))
            {
                //客户端 向 服务器 中的 服务 请求
                Proxy.OnMessage(this, pkgData);
            }
            else
            {
                //服务器 反向控制 客户端， 客户端 的服务回复
                if (!Clients.ContainsKey(pkgData.rcsn))
                {
                    //异常，没有发送过这个服务请求
                    pkgData.name = PkgName.ERR_NoService.ToString();

                    json = Newtonsoft.Json.JsonConvert.SerializeObject(pkgData);

                    SendTo(json);
                    return;
                }

                var client = Clients[pkgData.rcsn];
                client.OnMessage(pkgData);
            }
        }



        public override void SendTo(string msg)
        {
            if (IsConnected == false)
                return;
            base.Send(msg);
        }

        public void AddClient(string serviceName, WsServiceClient client)
        {
            client.SetSendEx(SendTo, serviceName);
            Clients.Add(serviceName, client);
            if (IsConnected)
            {
                client.OnOpen();
            }
        }

    }
}
